package com.ekexiu.project.storage.diacrisis;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.ImageWriter;
import javax.imageio.event.IIOReadWarningListener;
import javax.imageio.event.IIOWriteWarningListener;

import org.jfw.util.log.LogFactory;
import org.jfw.util.log.Logger;

import fr.apteryx.imageio.dicom.DicomReader;

public class AiFormatTask implements Runnable {
	private static Logger LOG = LogFactory.getLog(AiFormatTask.class);
	private String taskId;
	private int taskType;
	private AiTaskService aiTaskService;

	public AiFormatTask(String taskId, int taskType, AiTaskService aiTaskService) {
		this.taskId = taskId;
		this.taskType = taskType;
		this.aiTaskService = aiTaskService;
	}

	@Override
	public void run() {

		File file = new File(aiTaskService.getDstPath(), taskId);
		file = new File(file,"data_enhance");
		if (!file.exists()) {
			LOG.error("Path[" + file.getAbsolutePath() + "] not exists");
			aiTaskService.updateTaskState(taskId, 552, "ERR0403", null);
		} else {
			int begin, end;
			if (taskType == 1) {
				begin = 1;
				end = 7;
			} else if (taskType == 2) {
				begin = 7;
				end = 14;
			} else {
				LOG.error("err task type[" + taskType + "]");
				aiTaskService.updateTaskState(taskId, 552, "ERR0403", null);
				return;
			}
			for (int i = begin; i < end; ++i) {
				File src = new File(file, "IM_" + String.format("%04d.DICOM", i));
				if (!src.exists()) {
					LOG.error("Path[" + src.getAbsolutePath() + "] not exists");
					aiTaskService.updateTaskState(taskId, 552, "ERR0403", null);
					return;
				}
				File dst = new File(file, "ASD_" + String.format("%02d", i));
				if (dst.exists()) {
					LOG.error("Path[" + dst.getAbsolutePath() + "]  exists");
					aiTaskService.updateTaskState(taskId, 552, "ERR0403", null);
					return;
				}
				if (dst.mkdir()) {

					try {
						DicomReader reader = new DicomReader();
						reader.addIIOReadWarningListener(new WarningListener());
						reader.setInput(src);
						int nf = reader.getNumImages(true);
						if (nf <= 0) {
							LOG.error("file [" + src.getAbsolutePath() + "] invalid");
							aiTaskService.updateTaskState(taskId, 551, "ERR0402", null);
							return;
						}

						for (int k = 0; k < nf; ++k) {
							BufferedImage bi = reader.read(i);
							if (bi == null) {
								LOG.error("file [" + src.getAbsolutePath() + "] invalid");
								aiTaskService.updateTaskState(taskId, 551, "ERR0402", null);
								return;
							}

							File jpg = new File(dst, String.format("ASD_%02d_%03d.JPG", i, k));
							if (jpg.exists()) {
								LOG.error("file[" + jpg.getAbsolutePath() + "]  exists");
								aiTaskService.updateTaskState(taskId, 552, "ERR0403", null);
								return;
							}
							ImageIO.write(bi, "JPG", jpg);
						}
					} catch (Exception e) {
						aiTaskService.updateTaskState(taskId, 552, "ERR0403", null);
						return;
					}

				} else {
					LOG.error("makdir [" + dst.getAbsolutePath() + "] error");
					aiTaskService.updateTaskState(taskId, 552, "ERR0403", null);
					return;
				}
			}
		}
		aiTaskService.updateTaskState(taskId, 599, null, new AiDiacrisisTask(taskId, taskType, aiTaskService));
	}
}

class WarningListener implements IIOReadWarningListener, IIOWriteWarningListener {
	public void warningOccurred(ImageReader source, String warning) {
		System.err.println("READ WARNING: " + warning);
	}

	public void warningOccurred(ImageWriter source, int idx, String warning) {
		System.err.println("WRITE WARNING (image " + idx + "): " + warning);
	}
}
