package com.ekexiu.project.storage.diacrisis;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.concurrent.TimeUnit;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.PathVar;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.context.JfwAppContext;
import org.jfw.util.io.IoUtil;
import org.jfw.util.jdbc.JdbcUtil;
import org.jfw.util.log.LogFactory;
import org.jfw.util.log.Logger;

import com.ekexiu.project.storage.system.SessionUser;

@Path("/task")
public class AiTaskService {

	private static Logger LOG = LogFactory.getLog(AiTaskService.class);

	private long delayTime = 2000;

	@Autowrie
	private AiTaskDao aiTaskDao;
	private File srcPath = new File("/ekexiu/storage_asd/src");
	private File dstPath = new File("/ekexiu/storage_asd/dst");

	public File getSrcPath() {
		return srcPath;
	}

	public void setSrcPath(File srcPath) {
		this.srcPath = srcPath;
	}

	public File getDstPath() {
		return dstPath;
	}

	public void setDstPath(File dstPath) {
		this.dstPath = dstPath;
	}

	public AiTaskDao getAiTaskDao() {
		return aiTaskDao;
	}

	public void setAiTaskDao(AiTaskDao aiTaskDao) {
		this.aiTaskDao = aiTaskDao;
	}

	public boolean copyFile(File src, File dst) {
		try {
			if (src.exists() && (!dst.exists())) {
				InputStream in = null;
				OutputStream out = null;
				try {
					in = new FileInputStream(src);
					out = new FileOutputStream(dst);
					byte[] buf = new byte[4096];
					IoUtil.copy(in, out, buf);
					out.flush();
				} finally {
					if (in != null) {
						IoUtil.close(in);
					}
					if (out != null) {
						IoUtil.close(out);
					}
				}
				return true;
			}
		} catch (Exception e) {
		}
		return false;
	}

	public void updateTaskState(String id, int state, String stateCode, final Runnable runnable) {
		Connection con;
		try {
			con = JfwAppContext.getDataSource().getConnection();
			try {
				if (stateCode != null) {
					aiTaskDao.update(con, id, state, stateCode);
				} else {
					aiTaskDao.update(con, id, state);
				}
				con.commit();
				if (runnable != null) {
					JfwAppContext.getScheduledExecutorService().schedule(runnable, this.delayTime, TimeUnit.MILLISECONDS);
				}
			} catch (Exception e) {
				JdbcUtil.rollback(con);
			} finally {
				con.close();
			}
		} catch (Exception e1) {
		}
	}
	

   @Path("/add")
   @Get
   @Post
	public String insert(@JdbcConn(true) Connection con, @LoginUser SessionUser user, final String dataNum, final int taskType) throws SQLException {
		final String id = StringUtil.buildUUID();
		AiTask task = new AiTask();

		task.setDataNum(dataNum);
		task.setTaskType(taskType);
		task.setState(100);
		task.setStateCode(null);
		task.setId(id);
		task.setUid(user.getId());
		aiTaskDao.insert(con, task);
		
		JfwAppContext.getScheduledExecutorService().schedule(new Runnable() {
			@Override
			public void run() {
				updateTaskState(id, 200, null, new AiTransferTask(id, taskType,AiTaskService.this));
			}
		},this.delayTime,TimeUnit.MILLISECONDS);
		return id;
	}
   
   private static final String EMPTY_STRING_ARRAY[] = new String[0];
   @Get
   @Path("/pq")
   public PageQueryResult<AiTask> pageQuery(@JdbcConn Connection con,@LoginUser SessionUser user,@Nullable String dataNum,@Nullable Integer taskType,int pageSize,int pageNo) throws SQLException{
	   return aiTaskDao.pageQuery(con, user.getId(), dataNum, taskType, pageSize, pageNo);
   }
   @Get
   @Path("/result/{taskid}")
   public String[] getResult(@PathVar String taskid){
	   File path = new File(dstPath,taskid);
	   if(path.exists() && path.isDirectory()){
		   path = new File(path,"ASD_RESULT");
		   if(path.exists() && path.isDirectory()){
			   return path.list();
		   }
	   }
	   return EMPTY_STRING_ARRAY;
   }
   @Get
   @Path("/qo")
   public AiTask query(@JdbcConn Connection con,String id)throws SQLException{
	   return aiTaskDao.query(con, id);
   }
}
